# Generated by roxygen2: do not edit by hand

export(drop_empties)
export(scto_auth)
export(scto_catalog)
export(scto_get_attachments)
export(scto_get_form_definitions)
export(scto_get_form_metadata)
export(scto_get_form_responses)
export(scto_meta)
export(scto_quiet)
export(scto_read)
export(scto_unnest_form_definitions)
export(scto_write)
import(checkmate)
import(cli)
import(data.table)
importFrom(glue,glue)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(httr,cookies)
importFrom(httr,headers)
importFrom(httr,set_cookies)
importFrom(lifecycle,deprecated)
