% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scto_meta.R
\name{scto_meta}
\alias{scto_meta}
\alias{scto_catalog}
\title{Read metadata from a SurveyCTO server}
\usage{
scto_meta(auth)

scto_catalog(auth)
}
\arguments{
\item{auth}{\code{\link[=scto_auth]{scto_auth()}} object.}
}
\value{
\code{scto_meta()} returns a nested list of metadata related to forms,
datasets, groups, and publishing information. \code{scto_catalog()} returns a
\code{data.table} with one row per form or dataset.
}
\description{
These functions read metadata from a SurveyCTO server.
}
\examples{
\dontrun{
auth = scto_auth('scto_auth.txt')
metadata = scto_meta(auth)
catalog = scto_catalog(auth)
}

}
